@@ -12,7 +12,9 @@ package
 use strict;
 use warnings;
 
-sub DBIx::Class::Schema::load_namespaces {}
+no warnings 'redefine';
+
+*DBIx::Class::Schema::load_namespaces = sub {};
 
 
 package
@@ -21,12 +23,12 @@ package
 use strict;
 use warnings;
 
-sub DBIx::Class::load_components {}
-sub DBIx::Class::table {}
-sub DBIx::Class::add_columns {}
-sub DBIx::Class::set_primary_key {}
-sub DBIx::Class::belongs_to {}
-sub DBIx::Class::has_many {}
+*DBIx::Class::load_components = sub{};
+*DBIx::Class::table = sub {};
+*DBIx::Class::add_columns = sub{};
+*DBIx::Class::set_primary_key = sub{};
+*DBIx::Class::belongs_to = sub{};
+*DBIx::Class::has_many = sub{};
 
 
 1;
@@ -43,7 +45,7 @@ MySQL::Workbench::DBIC::FakeDBIC - provide some DBIx::Class method stubs when DB
 
 =head1 VERSION
 
-version 1.15
+version 1.16
 
 =head1 DESCRIPTION
 
